package java.awt;

import sun.awt.SunToolkit;
import sun.awt.ModalExclude;
import sun.awt.EventQueueDelegate;
import sun.awt.dnd.SunDragSourceContextPeer;
import java.util.ArrayList;
import sun.util.logging.PlatformLogger;

class EventDispatchThread extends Thread
{
    private static final PlatformLogger eventLog;
    private EventQueue theQueue;
    private volatile boolean doDispatch;
    private static final int ANY_EVENT = -1;
    private ArrayList<EventFilter> eventFilters;
    
    EventDispatchThread(ThreadGroup group, String name, EventQueue eventQueue) {
        super(group, name);
        this.doDispatch = true;
        this.eventFilters = new ArrayList();
        this.setEventQueue(eventQueue);
    }
    
    public void stopDispatching() {
        this.doDispatch = false;
    }
    
    @Override
    public void run() {
        try {
            this.pumpEvents(new Conditional() {
                @Override
                public boolean evaluate() {
                    return true;
                }
            });
        }
        finally {
            this.getEventQueue().detachDispatchThread(this);
        }
    }
    
    void pumpEvents(Conditional conditional) {
        this.pumpEvents(-1, conditional);
    }
    
    void pumpEventsForHierarchy(Conditional conditional, Component component) {
        this.pumpEventsForHierarchy(-1, conditional, component);
    }
    
    void pumpEvents(int n, Conditional conditional) {
        this.pumpEventsForHierarchy(n, conditional, null);
    }
    
    void pumpEventsForHierarchy(int n, Conditional conditional, Component component) {
        this.pumpEventsForFilter(n, conditional, new HierarchyEventFilter(component));
    }
    
    void pumpEventsForFilter(Conditional conditional, EventFilter eventFilter) {
        this.pumpEventsForFilter(-1, conditional, eventFilter);
    }
    
    void pumpEventsForFilter(int n, Conditional conditional, EventFilter eventFilter) {
        this.addEventFilter(eventFilter);
        this.doDispatch = true;
        while (this.doDispatch && !this.isInterrupted() && conditional.evaluate()) {
            this.pumpOneEventForFilters(n);
        }
        this.removeEventFilter(eventFilter);
    }
    
    void addEventFilter(EventFilter eventFilter) {
        if (EventDispatchThread.eventLog.isLoggable(PlatformLogger.Level.FINEST)) {
            EventDispatchThread.eventLog.finest("adding the event filter: " + eventFilter);
        }
        synchronized (this.eventFilters) {
            if (!this.eventFilters.contains(eventFilter)) {
                if ((eventFilter instanceof ModalEventFilter)) {
                    ModalEventFilter modalEventFilter = (ModalEventFilter)eventFilter;
                    int i = 0;
                    while (i < this.eventFilters.size()) {
                        EventFilter eventFilter2 = (EventFilter)this.eventFilters.get(i);
                        if (eventFilter2 instanceof ModalEventFilter && ((ModalEventFilter)eventFilter2).compareTo(modalEventFilter) > 0) {
                            break;
                        }
                        ++i;
                    }
                    this.eventFilters.add(i, eventFilter);
                }
                else {
                    this.eventFilters.add(eventFilter);
                }
            }
        }
    }
    
    void removeEventFilter(EventFilter eventFilter) {
        if (EventDispatchThread.eventLog.isLoggable(PlatformLogger.Level.FINEST)) {
            EventDispatchThread.eventLog.finest("removing the event filter: " + eventFilter);
        }
        synchronized (this.eventFilters) {
            this.eventFilters.remove(eventFilter);
        }
    }
    
    boolean filterAndCheckEvent(AWTEvent event) {
        boolean n = true;
        synchronized (this.eventFilters) {
            int i = this.eventFilters.size() - 1;
            while (i >= 0) {
                EventFilter.FilterAction acceptEvent = ((EventFilter)this.eventFilters.get(i)).acceptEvent(event);
                if (acceptEvent == EventFilter.FilterAction.REJECT) {
                    n = false;
                    break;
                }
                if (acceptEvent == EventFilter.FilterAction.ACCEPT_IMMEDIATELY) {
                    break;
                }
                --i;
            }
        }
        return n && SunDragSourceContextPeer.checkEvent(event);
    }
    
    void pumpOneEventForFilters(int n) {
        try {
            int filterAndCheckEvent;
            EventQueue eventQueue;
            EventQueueDelegate.Delegate delegate;
            AWTEvent nextEvent;
            do {
                eventQueue = this.getEventQueue();
                delegate = EventQueueDelegate.getDelegate();
                if (delegate != null && n == -1) {
                    nextEvent = delegate.getNextEvent(eventQueue);
                }
                else {
                    nextEvent = ((n == -1) ? eventQueue.getNextEvent() : eventQueue.getNextEvent(n));
                }
                filterAndCheckEvent = this.filterAndCheckEvent(nextEvent);
                if (filterAndCheckEvent == 0) {
                    nextEvent.consume();
                }
                else {
                    continue;
                }
            } while (filterAndCheckEvent == 0);
            if (EventDispatchThread.eventLog.isLoggable(PlatformLogger.Level.FINEST)) {
                EventDispatchThread.eventLog.finest("Dispatching: " + nextEvent);
            }
            Object beforeDispatch = null;
            if (delegate != null) {
                beforeDispatch = delegate.beforeDispatch(nextEvent);
            }
            eventQueue.dispatchEvent(nextEvent);
            if (delegate != null) {
                delegate.afterDispatch(nextEvent, beforeDispatch);
            }
        }
        catch (ThreadDeath threadDeath) {
            this.doDispatch = false;
            throw threadDeath;
        }
        catch (InterruptedException ex) {
            this.doDispatch = false;
        }
        catch (Throwable t) {
            this.processException(t);
        }
    }
    
    private void processException(Throwable obj) {
        if (EventDispatchThread.eventLog.isLoggable(PlatformLogger.Level.FINE)) {
            EventDispatchThread.eventLog.fine("Processing exception: " + obj);
        }
        this.getUncaughtExceptionHandler().uncaughtException(this, obj);
    }
    
    public synchronized EventQueue getEventQueue() {
        return this.theQueue;
    }
    
    public synchronized void setEventQueue(EventQueue theQueue) {
        this.theQueue = theQueue;
    }
    
    static {
        eventLog = PlatformLogger.getLogger("java.awt.event.EventDispatchThread");
    }
    
    private static class HierarchyEventFilter implements EventFilter
    {
        private Component modalComponent;
        
        public HierarchyEventFilter(Component modalComponent) {
            this.modalComponent = modalComponent;
        }
        
        @Override
        public EventFilter.FilterAction acceptEvent(AWTEvent awtEvent) {
            if (this.modalComponent != null) {
                int id = awtEvent.getID();
                int n = id >= 500 && id <= 507;
                int n2 = id >= 1001 && id <= 1001;
                int n3 = id == 201;
                if (Component.isInstanceOf(this.modalComponent, "javax.swing.JInternalFrame")) {
                    return n3 ? EventFilter.FilterAction.REJECT : EventFilter.FilterAction.ACCEPT;
                }
                if (n || n2 || n3) {
                    Object source = awtEvent.getSource();
                    if ((source instanceof ModalExclude)) {
                        return EventFilter.FilterAction.ACCEPT;
                    }
                    if ((source instanceof Component)) {
                        Component parent = (Component)source;
                        boolean n4 = false;
                        if ((this.modalComponent instanceof Container)) {
                            while (parent != this.modalComponent && parent != null) {
                                if (parent instanceof Window && SunToolkit.isModalExcluded((Window)parent)) {
                                    n4 = true;
                                    break;
                                }
                                parent = parent.getParent();
                            }
                        }
                        if (n4 == 0 && parent != this.modalComponent) {
                            return EventFilter.FilterAction.REJECT;
                        }
                    }
                }
            }
            return EventFilter.FilterAction.ACCEPT;
        }
    }
}
